// Copyright
//
// This software is Copyright (c) 1994 - 2008, Engenic Communications Solutions Corporation
// http://www.engenic.com
//
//
// License
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
// Executable code that uses provided source code, or derivative works thereof, may
// only be used in conjunction with TeleFlow.
//
// Redistributions of source code must retain the above copyright notice, this list
// of conditions and the following disclaimer.
//
// Redistributions in binary form must reproduce the above copyright notice, this
// list of conditions and the following disclaimer in the documentation, about box
// and/or other materials provided with the distribution.
//
// No personal names or organizations names associated with Engenic may be used to
// endorse or promote products derived from this software without specific prior
// written permission of the specific individual or organization.
//
//
// THIS SOFTWARE IS PROVIDED BY ENGENIC "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
// OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#ifndef LANGUAGE_MODULE__H
#define LANGUAGE_MODULE__H

#include <map>
using std::map;
#include <string>
using std::string;

// Class to simplify creating a comma delimited list of file names (words)
class WordList
{
   public:

      WordList(void);

      WordList& operator=(char character);
      WordList& operator=(string word);
      WordList& operator=(WordList list);

      // Append a character
      // e.g. "five,six" += 'S' -> "five,six,S"
      // e.g. "" += "S" -> "S"
      WordList& operator+=(char character);

      // Append a string
      // e.g. "five,six" += "seven" -> "five,six,seven"
      // e.g. "five,six" += "" -> "five,six"
      // e.g. "" += "seven" -> "seven"
      WordList& operator+=(string word);

      // Append a WordList
      // e.g. "five,six" += "seven,eight" -> "five,six,seven,eight"
      // e.g. "five,six" += "" -> "five,six"
      // e.g. "" += "seven,eight" -> "seven,eight"
      WordList& operator+=(WordList list);

      const char* c_str(void);
      string::size_type length(void);

   private:

      string wordList;
};

// -------------------------------------------------------------------------
// Class:       LanguageModule
//
// Purpose:     Every specific LanguageModule DLL must implement these functions
//
//              This generic definition allows TeleFlow Server to load any
//              DLL that implements a class that derives from this class.
//   -------------------------------------------------------------------------
class LanguageModule
{
	public:

      // Formats as specified in TeleFlow Designer
      enum SAY_FORMAT
      {
        SAY_DIGITS = 0,
        SAY_NUMBER = 1,
        SAY_DATE = 2,
        SAY_12HOUR = 3,
        SAY_MONEY = 4,
        SAY_ALPHA = 5,
        SAY_24HOUR = 6,
        SAY_YEARMONTH = 7,
        SAY_MONTHDAY = 8
      };

	   LanguageModule(void);

      // Creates a comma delimited list of file names (words) for play back
      // Allocates the returned string, caller is responsible for de-allocation
      // Caller is responsible to have text adhere to format
      // Returns empty string if text is NULL
      virtual char* BuildWordList(const char* text, LanguageModule::SAY_FORMAT format);

   protected:

      // Bases within the wordTable
      // e.g. cardinal word for 1 is wordTable[WB_CARDINAL + 1] ("one")
      // e.g. word for '#' is wordTable[WB_ASCII + '#'] ("pound")
      // e.g. ordinal for 1 is wordTable[WB_ORDINAL + 1] ("first")
      // e.g. month word for 1 is wordTable[WB_MONTH + 1] ("January")
      enum WORD_BASE
      {
         WB_CARDINAL     = 0,
         WB_ASCII        = 256,
         WB_ORDINAL      = 1000,
         WB_MONTH        = 2000,
      };

      // Each character is treated as a separate word
      // e.g. "1%45&" -> "one,percent,four,five,ampersand"
      virtual WordList AlphaNumericWordList(string text);

      // Each character is treated as a separate word but only digits are spoken
      // Non-digits are treated as pauses
      // e.g. "1%45&" -> "one,pause,four,five,pause"
      virtual WordList DigitsWordList(string digits);

      // Breaks an integer into groups of up to 3 digits separated by grouping words
      // e.g. "123,456,789" -> "one,hundred,and,twenty,three,million,four,hundred,
      // and,fifty,six,thousand,seven,hundred,and,eighty,nine"
      // This function calls the NumberGroupWordList methods of the derived class
      virtual WordList IntegerWordList(string number, bool ordinal = false);

      // Returns the portion of the number to the left of the decimal point,
      // potentially an empty string
      string Integer(string realNumber);

      // Returns the portion of the number to the right of the decimal point,
      // potentially an empty string
      string Fraction(string realNumber);

      virtual WordList DateWordList(string date, LanguageModule::SAY_FORMAT format) = 0;
      virtual WordList MoneyWordList(string money) = 0;

      // Takes a string of up to four characters, where the first character can
      // be either '+' or '-', and returns the words for the number
      // 0 will return an empty list
      // e.g. "-123" -> "negative,one,hundred,and,twenty,three"
      // e.g. "45" -> "forty,five"
      // e.g. "+780" -> "positive,seven,hundred,and,eighty"
      // e.g. "0" -> ""
      virtual WordList NumberGroupWordList(string numbergroup, bool ordinal) = 0;

      // Returns the grouping word for group index (max of 4)
      // 0 -> ""
      // 1 -> "thousand"
      // 2 -> "million"
      // 3 -> "billion"
      // 4 -> "trillion"
      virtual WordList NumberGroupWordList(int index, bool plural) = 0;

      // Returns the words for a real number
      // e.g. "123.456" -> "one,hundred,and,twenty,three,point,four,five,six"
      virtual WordList RealNumberWordList(string number) = 0;

      // Returns the words for time of the format HH:MM::SS or HH:MM in either
      // 12 or 24 hour format
      // e.g. "15:45:22", SAY_12HOUR -> "three,forty,five,and,twenty,two,seconds,pm"
      // e.g. "15:45", SAY_12HOUR -> "three,forty,five,pm"
      // e.g. "15:45:22", SAY_24HOUR -> "fifteen,forty,five,and,twenty,two,seconds"
      // e.g. "15:45", SAY_24HOUR -> "fifteen,forty,five"
      // e.g. "15:00", SAY_24HOUR -> "fifteen,hundred,hours"
      virtual WordList TimeWordList(string time, LanguageModule::SAY_FORMAT format) = 0;

      virtual void InitialiseWordTable(void) = 0;

      // Only numbers (based from WB_CARDINAL or WB_ORDINAL), months (based from
      // WB_MONTH), and ASCII characters from ' ' to '@', from '[' to '\'', from
      // '{' to '', '' or ''  (based on WB_ASCII) should be entered in wordTable
      map<int, string> wordTable;
};

// -------------------------------------------------------------------------
// Purpose:     These functions must be implemented by all board DLLs.
//              These functions are used to instantiate a board class
//              object.
// -------------------------------------------------------------------------

typedef LanguageModule* (*TCreateLanguageModule)(void);

#ifdef _MSC_VER
	extern "C" __declspec(dllexport) LanguageModule* CreateLanguageModule(void);
	extern "C" __declspec(dllexport) void ReleaseLanguageModule(LanguageModule* lang);
#else
	extern "C" LanguageModule* __declspec(dllexport) CreateLanguageModule(void);
	extern "C" void __declspec(dllexport) ReleaseLanguageModule(LanguageModule* lang);
#endif

typedef void (*TReleaseLanguageModule)(LanguageModule* lang);

#endif
