#ifndef XMLTFAPPPARSER_H
#define XMLTFAPPPARSER_H

#include <iostream>
using std::cout;
using std::cin;
using std::endl;

#include <string>
using std::string;

#include <vector>
using std::vector;

#include <sstream> //for string conversions

#define TIXML_USE_STL

// Warnings disabled
#pragma warn -8027
#include "..\3rdParty\tinyxml\tinyxml.h"
// Warnings enabled
#pragma warn +8027

#include "txtfile.h"
#include "..\TFServer\ivr_defs.h"
#include "step.h"

class XMLTFAppParser
{
   public:

      //XMLTFAppParser is initialised with the string fileName of the xml file. And loads the
      //file into a TiXmlDocument object.
      XMLTFAppParser(string fileName,Log* outLog);

      virtual ~XMLTFAppParser(void);

      //converts a string type to a MyChar type.
      void ConvertToMyChar(string data, MYCHAR& destination);  
   
      //LoadXmlSteps tries to get the root of the Xml file and calls LoadVectorSteps
      //to store a TiXmlNode pointer for every single Teleflow step.
      // 1 or 0 is returned on success or failure.
      int LoadXmlSteps(void);

      //Returns the number of TiXmlNode pointers loaded into the VectorStepNodes vector.
      //The number of teleflow steps loaded from the xml file is returned. 
      int StepSize(void);

      //Obtains any numeric attributes from a teleflow step.
      //-> 'int step' specifies which Teleflow step to access.
      //-> 'string record' specifies the attribute we want to obtain the value from
      //-> 'int *num' points to the value at the attribute.
      // 1 or 0 is returned on success or failure.
      int GetRecordNumeric(int step, string record, int* num);

      //Obtains any String type attribute from a teleflow step. Works in the same way
      //as GetRecordNumeric but only returns a string value 'val'.
      // 1 or 0 is returned on success or failure.
      int GetRecordString(int step,string record, string& val);

      //Obtains any data stored in a Numeric field of a teleflow step. Numeric fields in the Teleflow
      //Xml are specified by <N0x> and <N08> being the last numeric field.
      //-> 'int Step' specifies which teleflowstep to access.
      //-> 'int fieldIndex' specifies which variable field to access. A fieldIndex
      //    of 1 would the first field and an fieldIndex of 8 would be the last field.
      //-> 'int *num' will point to the value stored in the Numeric field.
      // 1 or 0 is returned on success or failure.
      int GetFieldNumeric(int step, int fieldIndex, int* num);

      //Obtains any data stored in a String field of teleflow step. String fields in
      //the Teleflow XML are specified by <s0x> and <s10> being the last string field.
      // 1 or 0 is returned on success or failure.
      int GetFieldString(int step, int fieldIndex, string& val);

      //Retrieves and checks if the parameter value exists
      // 1 or 0 is returned on success or failure.
      int GetParamValue(TiXmlNode* pParent,int paramIndex, string& values);

      //Retrieves the Variable or constant between the paramter field.
      // 1 or 0 is returned on success or failure.
      int GetParamName(TiXmlNode* pParent,int paramIndex,string& value);

      //Retrieves whether the variable is passed by reference or value.
      // 1 or 0 is returned on success or failure.
      int GetParamType(TiXmlNode* pParent, int paramIndex, int* num);

      //Get the description attribute value from the parameter tag.
      // 1 or 0 is returned on success or failure.
      int GetParamDescription(TiXmlNode* pParent, int paramIndex, string& value);

      //Checks wheter or not the logdisable field is set.
      //True or false is returned when the log is disabled or enabled. 
      bool GetLogDisabled(int step);

      //Returns true if no steps have been found otherwise returns false.
      //used to check if a file is a teleflow xml file or another type of file.
      int Empty(void);

      //Converts any type specified by source to a string type.
      template<class T>
      void ConvertToString(T source, string& destination);

      //Converts a string to the type specified.
      template<class T>
      void ConvertToNumeric(string source, T* destination);
      
   protected:

      // 1 or 0 is returned on success or failure.
      template<class T>
      int QueryXmlAttribute(TiXmlElement* pElement, string inputValue, T* returnValue);

      //A node is returned that corresponds to the teleflow step specified by 'int step'. 
      TiXmlNode* GetNode(int step);

      //LoadVectorSteps tries to store a TiXmlNode pointer. A pointer for every
      //teleflow step gets stored in the VectorStepNodes.
      //->TiXmlElement should point to the root of the xml teleflow file.
      // 1 or 0 is returned on success or failure.
      virtual int LoadVectorSteps(TiXmlElement* pElem);
 
      //Stores the DOM object created from the loaded Xml file.
      TiXmlDocument doc;

      //Stores the TiXmlNode pointers to every single Teleflow step in the xml file.
      vector<TiXmlNode*> VectorStepNodes;

      //Stores the filepath of the xml file.
      string filePath;

   private:

      Log *ivLog;
};
#endif
