#include <math.h>             // Needed for "ceil".
#include "step_vocpassauth.h"
#include "..\escutil\iv_str.h"
#include "..\BoardDLLs\brd_generic.h"
#include "..\Voice\Voice.h"
#include "VoiceCreator.h"

StepVocPassAuth::StepVocPassAuth(STEPREC& stepRecord) : Step(stepRecord)
{
	userID = stepRecord.s1.arr;
	authFactor = stepRecord.s5.arr;
   maxTime = stepRecord.s6.arr;
   disableBeep = stepRecord.u1;
}

StepVocPassAuth::~StepVocPassAuth(void)
{
}

const char* StepVocPassAuth::GetStepName(void) const
{
	return("Voice Pass Authorize");
}

IV_STATUS StepVocPassAuth::Execute(PIVRINFO pInfo)
{
   IV_STATUS status;						// status returned from this module

   // To evaluate parameters from the Step
   char* parsedUserID;  						// parsed version of user ID
	char* value;					// temporary string for conversion purposes
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   int iAuthFactor;				// authorization factor (allows for tight/loose setting)
   int iMaxTime;              // target maximum record length in milliseconds at step-level
                                    //  - this is the value supplied by the TeleFlow Designer
                                    //  - will get rounded to whole seconds before
                                    //    getting passed to board-level because
                                    //    VoiceCop expects value in seconds only

   // To perform voice authorization
   char*	cfgPath = pInfo->variables->FilePath("cfg\\eng\\");	// path to VoiceCop configuration files
   char* modelPath = pInfo->variables->FilePath("model\\");    // path to VoiceCop model files
   char*	audioBuf;	                           				    // Buffer to record audio password into
	unsigned int bufsize;			      		                   // size of audio buffer

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

	if(PortRequired(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FATAL);
   }

   // VoiceVerify will only function on licensed TeleFlow
   if(pInfo->portLicense == UNLICENSED)
   {
      pInfo->ivLog->Print ("      THIS PORT IS UNLICENSED\n");
      return(IVR_NOTSUPPORTED);
   }

	// Parse User ID field
   if((value = pInfo->variables->ParseParameter("User ID", userID.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
	   strtrim(value);
	   parsedUserID = pInfo->variables->FilePath(value);
	   delete[] value;
   }

	// Parse authorization factor
   if((value = pInfo->variables->ParseParameter("Authorization Factor", authFactor.c_str(), "50")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
	   iAuthFactor = atoi(value);
      delete[] value;

      if(iAuthFactor < 1)
      {
	      pInfo->ivLog->Print("      ERROR: Authorization factor below minimum of 1\n");
         bParmsValid = false;
      }
   	else if(iAuthFactor > 100)
      {
	      pInfo->ivLog->Print("      ERROR: Authorization factor above maximum of 100\n");
         bParmsValid = false;
      }
   }

   // Parse maximum record time
   if((value = pInfo->variables->ParseParameter("Record time", maxTime.c_str(), "4000")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iMaxTime = atoi(value);
      delete[] value;

      iMaxTime = (iMaxTime/1000 + (iMaxTime % 1000 > 0) ? 1 : 0)*1000;   // Rounded to the nearest second for VoiceCop
      pInfo->ivLog->Print("      Record time rounded to nearest second: %i ms\n", iMaxTime);
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   // Create new voice module if needed
   if(pInfo->voiceAuthenticationLibrary == NULL)
   {
      HINSTANCE library = GetModuleHandle("Voice_enGenic.dll");
      PVoiceCreator voiceCreator;

      if(library == NULL)
      {
         library = LoadLibrary("Voice_enGenic.dll");
      }

      voiceCreator = GetVoiceCreator(library);

      if(voiceCreator == NULL)
      {
         pInfo->ivLog->Print("      ERROR: Unable to access '%s'\n", "Voice_enGenic.dll");
         pInfo->ivLog->Print("         This feature requires Voice_enGenic.dll and special licensing.\n"
                             "         Please contact enGenic for more information.\n");
         return(IVR_NOTSUPPORTED);
      }

      pInfo->voiceAuthentication = voiceCreator(pInfo->iAppID, pInfo->ivLog, pInfo->ivBoard);

      if(pInfo->voiceAuthentication == NULL)
      {
         pInfo->ivLog->Print("      ERROR: Unable to create voice module\n");
         return(IVR_FATAL);
      }

      pInfo->voiceAuthenticationLibrary = library;
   }

	pInfo->ivLog->Print("         Config path = '%s'\n", cfgPath);
	pInfo->ivLog->Print("         Model path  = '%s'\n", modelPath);

	// Create record buffer
   bufsize = 8*2*iMaxTime;		// 8k samples per second, 2 byte samples

   // Create recording buffer
   audioBuf = new char[bufsize];
	memset(audioBuf, 0, bufsize);

   //
   // Record user speech
   // ------------------
   //
	pInfo->ivLog->Print("         Record to buffer \n");
	status = pInfo->ivBoard->AudioRecordBuffer(audioBuf,           // buffer to record into
	                                           bufsize,            // buffer size (bytes)
	                                           false,              // bool interruptable
	                                           TF_WAV_8KHZ_16BIT,  // SOUND_FORMAT format
	                                           0,                  // int volLevel, 0 = normal
	                                           iMaxTime,
	                                           disableBeep,	      // bool disableBeep
	                                           false               // bool automaticGainControl
	                                          );
   if(status != IVR_SUCCESS)
   {
   	pInfo->ivLog->Print("         ERROR: Voiceprint recording failed\n");
      delete[] audioBuf;
      return(status);
   }
	pInfo->ivLog->Print("         Record completed \n");

   if((status = pInfo->voiceAuthentication->VAStart(cfgPath, modelPath, bufsize)) != IVR_SUCCESS)
   {
      return(status);
   }

   if((status = pInfo->voiceAuthentication->VAAuthorise(parsedUserID, iAuthFactor, audioBuf, bufsize)) != IVR_SUCCESS)
   {
      return(status);
   }

   return(pInfo->voiceAuthentication->VAEnd());
}
