#include <stdlib.h>
#include <string.h>
#include "step_vocrecinit.h"
#include "..\BoardDLLs\brd_generic.h"
#include "VoiceCreator.h"

StepVocRecInit::StepVocRecInit(STEPREC& stepRecord) : Step(stepRecord)
{
   module = stepRecord.s1.arr;
   parameters = stepRecord.s6.arr;
   SREndOfSpeechTimeout = stepRecord.s2.arr;
   bargeIn = (bool) stepRecord.u1;
}

StepVocRecInit::~StepVocRecInit()
{
}

const char* StepVocRecInit::GetStepName(void) const
{
	return("SR Acquire");
}

IV_STATUS StepVocRecInit::Execute(PIVRINFO pInfo)
{
   char* value;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   string parsedParameters;
   unsigned int endOfSpeechTimeout;
   string moduleWithPath;
   HINSTANCE library;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

	if(PortRequired(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FATAL);
   }

   // Speech Recognition will only function on licensed TeleFlow
   if(pInfo->portLicense == UNLICENSED)
   {
      pInfo->ivLog->Print ("      THIS PORT IS UNLICENSED\n");
      return(IVR_NOTSUPPORTED);
   }

	// Process the package string
   if((value = pInfo->variables->ParseParameter("Speech Recognition Module", module.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      moduleWithPath = pInfo->pszInstallationPath;
      moduleWithPath += value;
      delete[] value;

      pInfo->ivLog->Print("      Module: '%s'\n", moduleWithPath.c_str());
   }

	// Process the package string
   if((value = pInfo->variables->ParseParameter("Speech Recognition Parameters", parameters.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedParameters = value;
      delete[] value;

      value = pInfo->variables->FilePath(parsedParameters.c_str());
      parsedParameters = value;
      delete[] value;
      
      pInfo->ivLog->Print("      Parameter(s): '%s'\n", parsedParameters.c_str());
   }

   // Get end of speech timeout
   if((value = pInfo->variables->ParseParameter("End of speech timeout", SREndOfSpeechTimeout.c_str(), "", "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      if(strlen(value) == 0)
      {
         endOfSpeechTimeout = pInfo->ivBoard->GetSREndOfSpeechTimeout();
         pInfo->ivLog->Print("      Empty end of speech defaults to current value '%d'\n", endOfSpeechTimeout);
      }
      else
      {
         endOfSpeechTimeout = atoi(value);
      }

      delete[] value;

      if(endOfSpeechTimeout <= 0)
      {
		   pInfo->ivLog->Print("      ERROR: End of speech timeout '%s' must be > 0\n", SREndOfSpeechTimeout.c_str());
         bParmsValid = false;
      }
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   library = GetModuleHandle(moduleWithPath.c_str());

   if(library == NULL)
   {
      library = LoadLibrary(moduleWithPath.c_str());
   }

   if(library == NULL)
   {
      DWORD errorID = GetLastError();
      char MsgBuf[1024];

      FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, errorID, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), MsgBuf, 1024, 0);

      pInfo->ivLog->Print("      ERROR: Unable to load '%s'\n", moduleWithPath.c_str());
      pInfo->ivLog->Print("         %s\n", MsgBuf);
      return(IVR_FAIL);
   }

   // Remove previous voice module
   if((pInfo->speechRecognitionLibrary != NULL) && (library != pInfo->speechRecognitionLibrary))
   {
      if((pInfo->speechRecognition != pInfo->voiceAuthentication) &&
         (pInfo->speechRecognition != pInfo->textToSpeech))
      {
         PVoiceDestroyer voiceDestroyer = GetVoiceDestroyer(pInfo->speechRecognitionLibrary);

         if(voiceDestroyer != NULL)
         {
            voiceDestroyer(pInfo->speechRecognition);
         }
      }
      pInfo->speechRecognition = NULL;
      pInfo->speechRecognitionLibrary = NULL;
   }

   // Create new voice module if needed
   if(pInfo->speechRecognitionLibrary == NULL)
   {
      PVoiceCreator voiceCreator = GetVoiceCreator(library);

      if(voiceCreator == NULL)
      {
         pInfo->ivLog->Print("      ERROR: Unable to access '%s'\n", moduleWithPath.c_str());
         return(IVR_FAIL);
      }

      pInfo->speechRecognition = voiceCreator(pInfo->iAppID, pInfo->ivLog, pInfo->ivBoard);

      if(pInfo->speechRecognition == NULL)
      {
         pInfo->ivLog->Print("      ERROR: Unable to create voice module\n");
         return(IVR_FAIL);
      }

      pInfo->speechRecognitionLibrary = library;
   }

   if(pInfo->speechRecognition->SRAcquire(parsedParameters.c_str(), pInfo->iPort, pInfo->pszAppNm, pInfo->pszLogDir, pInfo->logSRAudio) != IVR_SUCCESS)
   {
      pInfo->ivLog->Print("      ERROR: Unable to acquire voice recognition resource\n");
      return(IVR_FAIL);
   }

   pInfo->ivBoard->SetSREndOfSpeechTimeout(endOfSpeechTimeout);

   // Abuse of GetNextTAMName so that StepVocRecInit can update Speech Trainer
   pInfo->speechRecognition->SRUpdateTAMInfo(GetNextTAMName());

   if(bargeIn)
   {
      pInfo->ivLog->Print("      Barge-in enabled\n");
      return(pInfo->ivBoard->PermitBargeIn(true));
   }
   else
   {
      pInfo->ivLog->Print("      Barge-in disabled, operating in prompted mode only\n");
      return(pInfo->ivBoard->PermitBargeIn(false));
   }
}
