/* -------------------------------------------------------------------------
   File:        monevent.cpp

   Purpose:     Waits for events from the socket.  When the wait object is
                signalled, this thread sends a message to the primary thread
                to retrieve the message and process it.
   ------------------------------------------------------------------------- */
#include <vcl.h>
#include <stdlib.h>
#pragma hdrstop

#include "monevent.h"
#include "mon_main.h"

#pragma package(smart_init)

/* -------------------------------------------------------------------------
   Name:        SendData::SendData
   ------------------------------------------------------------------------- */
SendData::SendData (void *data, int dataSize) : Link()
{
   buf = new char [dataSize];
   memcpy (buf, data, dataSize);
   sendPtr = buf;
   remaining = dataSize;
}

/* -------------------------------------------------------------------------
   Name:        SendData::~SendData
   ------------------------------------------------------------------------- */
SendData::~SendData()
{
   delete [] buf;
}

/* -------------------------------------------------------------------------
   Name:        MonEvent::MonEvent                             (Constructor)

   Purpose:     Stores the handles passed in so they can be used by the
                Execute method.
   ------------------------------------------------------------------------- */
__fastcall MonEvent::MonEvent (char *pszHost, int port)
    : TThread(true)
{
   // Initialize Windows Socket API.
   NetInitialize();

   if(!NetConnect(&p, pszHost, port))
   {
      NetCloseSocket(&p);
   }

   // This event is used to close the TCP/IP socket.
   hCloseEvent  = CreateEvent (0, false, false, 0);

   hTimer = CreateWaitableTimer (0, false, 0);
   waitObjs[0] = hCloseEvent;        // Signalled when thread should terminate.
   waitObjs[1] = p.hEvent;           // Signalled when network event received.
   waitObjs[2] = hTimer;             // Signalled every sec. to update screen.

   memset (&readInfo, 0, sizeof (READ_INFO));
   readInfo.msgRemaining = sizeof (NET_MSG);
   readInfo.msgPtr = &readInfo.msg;
   bReconnect = true;

   host = new char[strlen(pszHost) + 1];
   strcpy(host, pszHost);
}
/*MonEvent::MonEvent*/

/* -------------------------------------------------------------------------
   Name:        MonEvent::~MonEvent                             (Destructor)

   Purpose:     There is a bug in Borland's NT libraries that prevents the
                primary thread from deleting this class instance.  The
                primary thread should wait until the thread terminates
                before processing other connection requests.  When Borland's
                libraries finally get around to deleting this class, this
                function will post a message to the primary thread so it
                knows that this thread has really terminated.
   ------------------------------------------------------------------------- */
__fastcall MonEvent::~MonEvent()
{
   CloseHandle (hTimer);
   CloseHandle (hCloseEvent);

   NetCloseSocket (&p);
   // Allow Windows Socket API to cleanup.
   NetShutdown();
   Synchronize (SayGoodnight);

   delete host;
}
/*MonEvent::~MonEvent*/

/* -------------------------------------------------------------------------
   ------------------------------------------------------------------------- */
void __fastcall MonEvent::SayGoodnight()
{
   FormMon->bPleaseReconnect = bReconnect;
   FormMon->ThreadDone();
}

/* -------------------------------------------------------------------------
   ------------------------------------------------------------------------- */
void __fastcall MonEvent::ConnectionFailed()
{
	FormMon->threadColour = clRed;
   FormMon->SetPanelMessage ("Unable to contact server.");
   FormMon->SetConnectedToHost(false);
   SetEvent(hCloseEvent);
}

/* -------------------------------------------------------------------------
   ------------------------------------------------------------------------- */
void __fastcall MonEvent::ConnectionOpen()
{
	FormMon->threadColour = clLime;
   FormMon->SetPanelMessage ("Connected to server.");
   FormMon->SetConnectedToHost(true);
   FormMon->EnableSocketMenu (false);
   FormMon->Caption = "TeleFlow Monitor - " + (AnsiString) host;
}

/* -------------------------------------------------------------------------
   Name:        MonEvent::Execute

   Purpose:     Waits on for network events to be received or until it is
                told to terminate.
   ------------------------------------------------------------------------- */
void __fastcall MonEvent::Execute()
{
   DWORD          ret;
   int            event;
   long           timeout = 2;  // Measured in seconds.

   if (p.hSocket == INVALID_SOCKET)
      return;

   waitTimeout.QuadPart = -(LONGLONG)timeout * 10000000;

   for (;;) {
      ret = WaitForMultipleObjects (3, waitObjs, false, INFINITE);

      if (ret == WAIT_FAILED) {            // Something is wrong, bail out.
         Synchronize (ConnectionFailed);
         break;
      }
      else if (ret == WAIT_OBJECT_0) {     // Thread should terminate.
         bReconnect = false;
         break;
      }
      else if (ret == WAIT_OBJECT_0 + 1) { // Network event has been received.
         if (!NetGetEvent (&p, event)) {
            Synchronize (ConnectionFailed);
            break;
         }

         if (event == NET_EVT_CLOSED) {
            Synchronize (ConnectionFailed);
            break;
         }
         else if (event == NET_EVT_OPEN) {
            Synchronize (ConnectionOpen);
            SetWaitableTimer (hTimer, &waitTimeout, 0, 0, 0, false);
            bReplyPending = false;
         }
         else if (event == NET_EVT_READ) {
            if (!HandleRead()) {
               break;
            }
         }
         else if (event == NET_EVT_WRITE) {
            if (!HandleWrite()) {
               break;
            }
         }
      }
      else if (ret == WAIT_OBJECT_0 + 2) { // Timer expired.
         if (!bReplyPending) {
            bReplyPending = true;
            SetWaitableTimer (hTimer, &waitTimeout, 0, 0, 0, false);
            RequestUpdate (false);
            HandleWrite();
         }
      }
   }

   FormMon->SetConnectedToHost(false);
}
/*MonEvent::Execute*/

/* -------------------------------------------------------------------------
   Name:        MonEvent::RequestUpdate
   ------------------------------------------------------------------------- */
void __fastcall MonEvent::RequestUpdate (bool bRefresh)
{
   SendData  *myData;
   NET_MSG    msg;

   msg.id       = NET_EVT_SEND_UPDATE;
   msg.value    = (unsigned long) bRefresh;
   msg.data     = 0;
   msg.dataSize = 0;
   myData = new SendData (&msg, sizeof (NET_MSG));
   myQueue.AddLast ((Link*)myData);
}
/*MonEvent::RequestUpdate*/

/* -------------------------------------------------------------------------
   Name:        MonEvent::HandleRead
   ------------------------------------------------------------------------- */
bool __fastcall MonEvent::HandleRead()
{
   int   bytesRcvd;
   int   err;
   bool  ret;

   // ---------------------------------------------------- READ IN THE NET_MSG.
   if (readInfo.msgRemaining > 0) {
      bytesRcvd = recv (p.hSocket,
                        (char*)readInfo.msgPtr,
                        readInfo.msgRemaining,
                        0);
      if (bytesRcvd == SOCKET_ERROR) {
         err = WSAGetLastError();
         if (err == WSAEWOULDBLOCK)
            return true;   // This should never happen, but handle it anyway.
         else
            return false;  // Something went wrong, connection should close.
      }
      readInfo.msgRemaining -= bytesRcvd;
      readInfo.msgPtr        = ((char*)readInfo.msgPtr) + bytesRcvd;
      if (readInfo.msgRemaining == 0) {
         readInfo.dataRemaining = readInfo.msg.dataSize;
         if (readInfo.msg.dataSize > 0) {
            readInfo.clientData = new char [readInfo.msg.dataSize];
            readInfo.dataPtr = readInfo.clientData;
            return true;   // Have NET_MSG, now wait for data.
         }
      } else
         return true;      // Still waiting for more data for NET_MSG.
   }

   // -------------------------------------------------------------------------
   // If there is data attached to the NET_MSG structure,
   // wait for the data to arrive.  If the message does not
   // contain any data, skip over this section and process
   // the message.
   // ------------------------------------------------------- READ IN THE DATA.
   if (readInfo.dataRemaining > 0) {
      bytesRcvd = recv (p.hSocket,
                        (char*)readInfo.dataPtr,
                        readInfo.dataRemaining,
                        0);
      if (bytesRcvd == SOCKET_ERROR) {
         err = WSAGetLastError();
         if (err == WSAEWOULDBLOCK)
            return true;   // This should never happen, but handle it anyway.
         else
            return false;  // Something went wrong, connection should close.
      }
      readInfo.dataRemaining -= bytesRcvd;
      readInfo.dataPtr        = ((char*)readInfo.dataPtr) + bytesRcvd;
      if (readInfo.dataRemaining > 0)
         return true;      // Still waiting for more data for NET_MSG.
   }

   // ---------------------------------------------------- PROCESS THE MESSAGE.
   switch (readInfo.msg.id)
   {
   case NET_EVT_UPDATE:
      Synchronize (ProcessUpdate);
      SetWaitableTimer (hTimer, &waitTimeout, 0, 0, 0, false);
      bReplyPending = false;
      ret = true;
      break;

   case NET_EVT_OUTPUT:
      Synchronize (ProcessMessages);
      ret = true;
      break;

   default:
      ret = false;
   }

   if (ret)
      ret = HandleWrite();

   // Reset for next message.
   readInfo.msgPtr = (void*) &readInfo.msg;
   readInfo.msgRemaining = sizeof(NET_MSG);
   delete [] readInfo.clientData;
   readInfo.clientData = 0;
   readInfo.dataPtr = 0;
   readInfo.dataRemaining = 0;

   return ret;
}
/*MonEvent::HandleRead*/

/* -------------------------------------------------------------------------
   Name:        MonEvent::HandleWrite
   ------------------------------------------------------------------------- */
bool __fastcall MonEvent::HandleWrite()
{
   SendData  *myData;
   SendData  *tempData;
   int        bytesSent;
   int        err;

   myData = (SendData*) myQueue.First();

   while (myData) {
      bytesSent = send (p.hSocket, myData->sendPtr, myData->remaining, 0);
      if (bytesSent == SOCKET_ERROR) {
         err = WSAGetLastError();
         if (err == WSAEWOULDBLOCK)
            return true;   // Buffer full, wait for FD_WRITE.
         else {
            DisplayError (err);
            return false;  // Something went wrong, connection should close.
         }
      }
      myData->remaining -= bytesSent;
      myData->sendPtr   += bytesSent;
      if (myData->remaining == 0) {
         tempData = myData;
         myData = (SendData*) myData->Suc();
         delete tempData;
      }
   }
   return true;
}
/*MonEvent::HandleWrite*/

/* -------------------------------------------------------------------------
   Name:        MonEvent::ProcessUpdate
   ------------------------------------------------------------------------- */
void __fastcall MonEvent::ProcessUpdate()
{
   unsigned      rows;
   unsigned      i;
   unsigned      index;
   CLIENT_MSG   *msg;

   if (readInfo.msg.dataSize)
      rows = readInfo.msg.dataSize / sizeof (CLIENT_MSG);
   else
      rows = 0;
   msg = (CLIENT_MSG*) readInfo.clientData;

   for (i = 0; i < rows; i++) {
      index = 0;
      while (FormMon->clientMsgs[index].status != lsEmpty     &&
             FormMon->clientMsgs[index].appID != msg[i].appID &&
             index < MAX_CLIENTS)
         index++;

      // Either we are updating an existing slot or adding a new one.
      // There should be no empty slots in the middle of the list.
      if (index < MAX_CLIENTS) {
         FormMon->clientMsgs[index].status = msg[i].status;
         FormMon->clientMsgs[index].appID  = msg[i].appID;
			// For .TAL file name transfer
			strncpy(FormMon->clientMsgs[index].appFile, msg[i].appFile, TELEFLOW_TAP_NAME_MAX);
			FormMon->clientMsgs[index].appFile[TELEFLOW_TAP_NAME_MAX] = '\0';
         FormMon->clientMsgs[index].stepNo = msg[i].stepNo;
         FormMon->clientMsgs[index].lineNo = msg[i].lineNo;
         strncpy(FormMon->clientMsgs[index].appNm, msg[i].appNm, TELEFLOW_APP_NAME_MAX);
			FormMon->clientMsgs[index].appNm[TELEFLOW_APP_NAME_MAX] = '\0';
         strcpy(FormMon->clientMsgs[index].currentTAMName, msg[i].currentTAMName);
         strcpy(FormMon->clientMsgs[index].stepName, msg[i].stepName);
         strcpy(FormMon->clientMsgs[index].stepData, msg[i].stepData);
         FormMon->clientMsgs[index].restartCnt = msg[i].restartCnt;
         FormMon->clientMsgs[index].restartMax = msg[i].restartMax;
         FormMon->InvalidateRow(index + 1);

			// For packet logs
         if(FormMon->bLogPackets)
         {
				int logIndex = FormMon->appLogs[index].index;
				strncpy(FormMon->appLogs[index].log[logIndex].currentTAMName, msg[i].currentTAMName, 30);
				FormMon->appLogs[index].log[logIndex].currentTAMName[30] = '\0';
				FormMon->appLogs[index].log[logIndex].stepNo = msg[i].stepNo;
				strncpy(FormMon->appLogs[index].log[logIndex].stepName, msg[i].stepName, 30);
				FormMon->appLogs[index].log[logIndex].stepName[30] = '\0';
				FormMon->appLogs[index].log[logIndex].lineNo = msg[i].lineNo;

				// Increment index (of log) and cycle back to 0 at INFOLOG_MAX
	         if(++(FormMon->appLogs[index].index) == INFOLOG_MAX)
   	      {
      	   	FormMon->appLogs[index].index = 0;
					FormMon->appLogs[index].logWrapped = true;
	         }
   	   }
      }
   }

   // Resize the grid to match the entries
   FormMon->ResizeGrid(rows);
}
/*MonEvent::ProcessUpdate*/

/* -------------------------------------------------------------------------
   Name:        MonEvent::ProcessMessages
   ------------------------------------------------------------------------- */
void __fastcall MonEvent::ProcessMessages()
{
   FormMon->MsgDisplayLines (readInfo.msg.value, (char*)readInfo.clientData);
}
/*MonEvent::ProcessMessages*/

/* -------------------------------------------------------------------------
   Name:        MonEvent::DisplayError
   ------------------------------------------------------------------------- */
void __fastcall MonEvent::DisplayError (int error)
{
   iError = error;
   Synchronize (SyncDisplayError);
}
/*MonEvent::DisplayError*/

/* -------------------------------------------------------------------------
   Name:        MonEvent::SyncDisplayError
   ------------------------------------------------------------------------- */
void __fastcall MonEvent::SyncDisplayError()
{
   char *szCode;

   switch (iError)
   {
   // Windows Sockets definitions of regular Microsoft C error constants
   case WSAEINTR:               szCode = "WSAEINTR";               break;
   case WSAEBADF:               szCode = "WSAEBADF";               break;
   case WSAEACCES:              szCode = "WSAEACCES";              break;
   case WSAEFAULT:              szCode = "WSAEFAULT";              break;
   case WSAEINVAL:              szCode = "WSAEINVAL";              break;
   case WSAEMFILE:              szCode = "WSAEMFILE";              break;
   // Windows Sockets definitions of regular Berkeley error constants
   case WSAEWOULDBLOCK:         szCode = "WSAEWOULDBLOCK";         break;
   case WSAEINPROGRESS:         szCode = "WSAEINPROGRESS";         break;
   case WSAEALREADY:            szCode = "WSAEALREADY";            break;
   case WSAENOTSOCK:            szCode = "WSAENOTSOCK";            break;
   case WSAEDESTADDRREQ:        szCode = "WSAEDESTADDRREQ";        break;
   case WSAEMSGSIZE:            szCode = "WSAEMSGSIZE";            break;
   case WSAEPROTOTYPE:          szCode = "WSAEPROTOTYPE";          break;
   case WSAENOPROTOOPT:         szCode = "WSAENOPROTOOPT";         break;
   case WSAEPROTONOSUPPORT:     szCode = "WSAEPROTONOSUPPORT";     break;
   case WSAESOCKTNOSUPPORT:     szCode = "WSAESOCKTNOSUPPORT";     break;
   case WSAEOPNOTSUPP:          szCode = "WSAEOPNOTSUPP";          break;
   case WSAEPFNOSUPPORT:        szCode = "WSAEPFNOSUPPORT";        break;
   case WSAEAFNOSUPPORT:        szCode = "WSAEAFNOSUPPORT";        break;
   case WSAEADDRINUSE:          szCode = "WSAEADDRINUSE";          break;
   case WSAEADDRNOTAVAIL:       szCode = "WSAEADDRNOTAVAIL";       break;
   case WSAENETDOWN:            szCode = "WSAENETDOWN";            break;
   case WSAENETUNREACH:         szCode = "WSAENETUNREACH";         break;
   case WSAENETRESET:           szCode = "WSAENETRESET";           break;
   case WSAECONNABORTED:        szCode = "WSAECONNABORTED";        break;
   case WSAECONNRESET:          szCode = "WSAECONNRESET";          break;
   case WSAENOBUFS:             szCode = "WSAENOBUFS";             break;
   case WSAEISCONN:             szCode = "WSAEISCONN";             break;
   case WSAENOTCONN:            szCode = "WSAENOTCONN";            break;
   case WSAESHUTDOWN:           szCode = "WSAESHUTDOWN";           break;
   case WSAETOOMANYREFS:        szCode = "WSAETOOMANYREFS";        break;
   case WSAETIMEDOUT:           szCode = "WSAETIMEDOUT";           break;
   case WSAECONNREFUSED:        szCode = "WSAECONNREFUSED";        break;
   case WSAELOOP:               szCode = "WSAELOOP";               break;
   case WSAENAMETOOLONG:        szCode = "WSAENAMETOOLONG";        break;
   case WSAEHOSTDOWN:           szCode = "WSAEHOSTDOWN";           break;
   case WSAEHOSTUNREACH:        szCode = "WSAEHOSTUNREACH";        break;
   case WSAENOTEMPTY:           szCode = "WSAENOTEMPTY";           break;
   case WSAEPROCLIM:            szCode = "WSAEPROCLIM";            break;
   case WSAEUSERS:              szCode = "WSAEUSERS";              break;
   case WSAEDQUOT:              szCode = "WSAEDQUOT";              break;
   case WSAESTALE:              szCode = "WSAESTALE";              break;
   case WSAEREMOTE:             szCode = "WSAEREMOTE";             break;
   // Extended Windows Sockets error constant definitions
   case WSASYSNOTREADY:         szCode = "WSASYSNOTREADY";         break;
   case WSAVERNOTSUPPORTED:     szCode = "WSAVERNOTSUPPORTED";     break;
   case WSANOTINITIALISED:      szCode = "WSANOTINITIALISED";      break;
   case WSAEDISCON:             szCode = "WSAEDISCON";             break;
   case WSAENOMORE:             szCode = "WSAENOMORE";             break;
   case WSAECANCELLED:          szCode = "WSAECANCELLED";          break;
   case WSAEINVALIDPROCTABLE:   szCode = "WSAEINVALIDPROCTABLE";   break;
   case WSAEINVALIDPROVIDER:    szCode = "WSAEINVALIDPROVIDER";    break;
   case WSAEPROVIDERFAILEDINIT: szCode = "WSAEPROVIDERFAILEDINIT"; break;
   case WSASYSCALLFAILURE:      szCode = "WSASYSCALLFAILURE";      break;
   case WSASERVICE_NOT_FOUND:   szCode = "WSASERVICE_NOT_FOUND";   break;
   case WSATYPE_NOT_FOUND:      szCode = "WSATYPE_NOT_FOUND";      break;
   case WSA_E_NO_MORE:          szCode = "WSA_E_NO_MORE";          break;
   case WSA_E_CANCELLED:        szCode = "WSA_E_CANCELLED";        break;
   case WSAEREFUSED:            szCode = "WSAEREFUSED";            break;

   // Error return codes from gethostbyname and gethostbyaddr.
   case WSAHOST_NOT_FOUND:      szCode = "WSAHOST_NOT_FOUND";      break;
   case WSATRY_AGAIN:           szCode = "WSATRY_AGAIN";           break;
   case WSANO_RECOVERY:         szCode = "WSANO_RECOVERY";         break;
   case WSANO_DATA:             szCode = "WSANO_DATA";             break;
// case WSANO_ADDRESS:  Maps to WSANO_DATA.
   default:                     szCode = "unknown";
   }

   FormMon->DisplayError (szCode);
}
/*MonEvent::SyncDisplayError*/

