#ifndef VOICE__H
#define VOICE__H

#include "..\TFServer\ivr_defs.h"
#include "..\TFServer\log.h"
#include "..\BoardDLLs\brd_generic.h"
#include "..\escutil\ESCTime.h"

typedef struct
{
   string TAPName;
   string TAMName;
   int    stepNum;
   string stepName;
} STEPSTATISTIC;

typedef struct
{
   STEPSTATISTIC  stepStat;
   string         script;
   string         file;
   bool           completed;
   float          played_sec;
} PLAYSTATISTIC;

class Voice
{
   public:

      enum SR_GRAMMAR
      {
         FILE,
         YES_NO,
         DIGITS,
         TEXT
      };

      Voice(unsigned int applid, Log* ivLog, BoardGeneric* board);
      virtual ~Voice(void);

      virtual IV_STATUS SRAcquire(const char* srResource, int telephonyPort, const char* pszAppName, const char* pszLogDir, bool logSRAudio);

      virtual IV_STATUS SRAddGrammar(Voice::SR_GRAMMAR grammarType, const char* grammar);

      // Preserve statistical info about Play steps
      // Parameters are passed in from the step.execute code
      // Some values are retrieved from the audioState class of the board
      // Data is manipulated in the SpeechTrainer private properties
      virtual void SRCapturePlayStats(const char* pszAppFile,      // TAP name
                            int iStepNum,                // step number
                            const char* pszStepName,     // Step name
                            const char* pszScriptcode,   // Script code
                            const char* pszFileName);    // File/path name to audio file

      // Capture SR result for SpeechTrainer
      virtual void SRCaptureResult(const char cDig,               // touch tone digit
                           const char* pszConcept,        // result of recognition
                           unsigned int iConfidence,      // confidence
                           IV_STATUS status);             // result

      // Preserve statistical info about SR steps
      // Parameters are passed in from the step.execute code
      // Data is manipulated in the SpeechTrainer private properties
      virtual void SRCaptureStartStats(const char* pszAppFile,        // TAP name
                               int iStepNum,                  // step number
                               const char* pszStepName,       // Step name
                               int iNoVoiceTimeout,           // no voice t/o
                               unsigned int iEndVoiceTimeout, // end speech t/o
                               unsigned int iThresholdConf,   // threshold confidence
                               bool bRetainAudio,             // retainAudio
                               Voice::SR_GRAMMAR srgType,     // grammar type
                               const char* pszBaseGrammar,    // base grammar name/file
                               const char* pszAddGrammar);    // additional grammar text

      // Sets SRIsPendingAudio() result to false
      // Sets SRIsPendingResult() result to false
      // Sets SRIsReceivingAudio() result to false
      // Clears handle for SRGetRecognitionHandle()
      virtual IV_STATUS SRClearAudio(void);

      // Virtual method for each SR client to perform final clean up after recognition
      // is completed or cancelled
      virtual IV_STATUS SREnd(void);

      // Flag a previous SR result for review in SpeechTrainer GUI application
      virtual bool SRFlagResult(const char* SRHandle);

      // Returns the starting address of the internal audio buffer
      virtual char* SRGetAudioBuffer(void);

      // Returns the size in bytes of the internal audio buffer
      virtual unsigned int SRGetAudioBufferLength(void);

      // Returns the size in bytes of a 200 ms buffer using the audio format specified
      // by SRSetAudioFormat()
      virtual unsigned int SRGetAudioChunkSize(void);

      // Returns the address of the next available 200 ms buffer using the audio
      // format specified by SRSetAudioFormat()
      virtual char* SRGetNextAudioChunk(void);

      // Returns a unique identifier for the current recognition
      // This handle is cleared by SRClearAudio()
      virtual const char* SRGetRecognitionHandle(void);

      // Copies up to maxlength characters of the client recognition result to concept
      // Sets confidence to a the client recognition result confidence percentage (0 to 100)
      virtual IV_STATUS SRGetResult(char* concept, unsigned int maxlength, unsigned int& confidence);

      // Returns true if there is SR audio waiting to be recognised
      // Note that SRIsReceivingAudio() will be false if SRIsPendingAudio() is true
      virtual bool SRIsPendingAudio(void);

      // Returns true if the recognition result is being processed
      // Note that SRIsReceivingAudio() will be false if SRIsPendingResult() is true
      // Note that SRStartAudio() should not be called while SRIsPendingResult() is true
      virtual bool SRIsPendingResult(void);

      // Returns true if SR audio has begun but not finished
      // Note that SRIsPendingAudio() will be false if SRIsReceivingAudio() is true
      // Note that SRIsPendingResult() will be false if SRIsReceivingAudio() is true
      virtual bool SRIsReceivingAudio(void);

      virtual IV_STATUS SRLoadGrammar(Voice::SR_GRAMMAR grammarType, const char* grammar);

      virtual IV_STATUS SRRelease(void);

      // Sets the format of the audio that the SR client will receive
      // Determines the size of the audio chunk returned by SRGetNextAudioChunk()
      // Determines the size returned by SRGetAudioChunkSize()
      virtual void SRSetAudioFormat(SOUND_FORMAT audioFormat);

      // Sets the length of SR audio in the internal buffer starting at the address
      // specified by SRSetAudioStart()
      virtual void SRSetAudioLength(unsigned int length);

      // Sets the start of SR audio in the internal buffer
      virtual void SRSetAudioStart(const char* start);

      // Sets SRIsPendingAudio() result to false
      // Sets SRIsPendingResult() result to false
      // Sets SRIsReceivingAudio() result to true
      // Clears handle for SRGetRecognitionHandle()
      // Sets the SR audio start time
      virtual IV_STATUS SRStartAudio(void);

      // Sets SRIsPendingResult() result to true
      // Sets SRIsReceivingAudio() result to false
      // Sets the SR audio stop time
      // Makes a copy of the SR audio so that a new recognition may begin
      virtual IV_STATUS SRStopAudio(void);

      // Update the current TAM (full) name in the SpeechTrainer statistics
      virtual void SRUpdateTAMInfo(const char* tamName);

      // Intended for use with streaming SR clients
      virtual IV_STATUS SRValidateAudio(const char* audiobuf, unsigned int audiobuflen);

      virtual IV_STATUS VAAcquire(const char* vaResource);

      virtual IV_STATUS VARelease(void);

      virtual IV_STATUS VAAuthorise(const char* userID, int authFactor, const char* audioBuf, int bufferSize);

      virtual IV_STATUS VAEnd(void);

      virtual IV_STATUS VARegister(const char* userID);

      virtual IV_STATUS VASubmit(int authFactor, const char* audioBuf, int bufferSize);

      virtual IV_STATUS VAStart(const char* cfgPath, const char* modelPath, int bufferSize);

   protected:

      unsigned int applid;
      Log* ivLog;
      BoardGeneric* ivBoard;
      bool isPendingSRAudio;
      bool isPendingSRResult;
      bool isReceivingSRAudio;
      unsigned int nextSRAudioBufferIndex;
      unsigned int SRAudioBeginBufferIndex;
      unsigned int SRAudioLength;
      SOUND_FORMAT SRAudioFormat;
      // 10 s * 8000 Hz * 2 bytes/sample (TF_WAV_8KHZ_16BIT)
      enum CONSTANT { AUDIO_MAX = 10*8000*2 };
      char incomingSRAudio[AUDIO_MAX];
      char outgoingSRAudio[AUDIO_MAX];
      ESCTime SRAudioStartTime;
      ESCTime SRAudioEndTime;
      string srResource;       // parameter string passed to recognizer upon SR Acquire

      // Speech Trainer data
      bool logSRAudio;                  // SpeechTrainer needs to know if SR logging on/off
      int telephonyPort;                // port number
      string appName;                   // application name, as identified in TAL file
      string logDir;                    // current TeleFlow log directory
      string currentTAMName;            // full path+name of current TAM
      PLAYSTATISTIC lastPlayedStep;
      PLAYSTATISTIC lastExecPlayStep;
      STEPSTATISTIC lastSRStep;
      unsigned int novoiceTimeout;
      unsigned int endvoiceTimeout;
      unsigned int thresholdConf;
      bool retainAudio;
      Voice::SR_GRAMMAR grammarType;
      string baseGrammarIdentifier;
      string baseGrammarFileName;
      string additionalGrammarFileName;
      string SRHandle;
      string SRHeard;
      string SRAdjusted;
      unsigned int SRConfidence;
      IV_STATUS SRReturn;
      char termDigit;              // terminating digit, if SR terminated by a key press

      virtual string SRCreateHandle(void);

      virtual string SRGetAudioFileName(string handle = "") const;

      virtual string SRGetAdditionalGrammarFileName(string handle = "") const;

      virtual string SRGetBaseGrammarFileName(string handle = "") const;

      virtual unsigned int SRGetHandleLength(void) const;

      virtual string SRGetTrainerDataName(string handle = "") const;

      // Returns the full path where speech trainer files should be written
      // creates the folder if it does not already exists
      // returns an empty string if create fails
      virtual string SRGetTrainerPath(string handle = "") const;

      // Clears properties to reset Speech Trainer for new recognition
      // Called by constructor and by WriteSRData after SR stats have been reported to file
      virtual void SRInvalidateStatistics(void);

      virtual IV_STATUS SRWriteAudioToFile(const char* audioBuf, unsigned int bytes);

      // Write to trainer file
      virtual void SRWriteData(void);
};

#ifdef _MSC_VER
	extern "C" __declspec(dllexport) Voice* CreateVoice(unsigned int applid, Log* ivLog, BoardGeneric* board);
	extern "C" __declspec(dllexport) void DestroyVoice(Voice* voiceModule);
#else
	extern "C" Voice* __declspec(dllexport) CreateVoice(unsigned int applid, Log* ivLog, BoardGeneric* board);
	extern "C" void __declspec(dllexport) DestroyVoice(Voice* voiceModule);
#endif

#endif
