/* -------------------------------------------------------------------------
   File:        parmlist.cpp

   Notes:       - The string to be used as a parameter list must be parsed
                  before calling this function.
   ------------------------------------------------------------------------- */
#include <stdio.h>	// printf
#include <string.h>	// strcmp, memcpy, strchr, strlen, strcpy

#include "iv_str.h"	// strclear, charadd, stradd
#include "parmlist.h"

/* -------------------------------------------------------------------------
   Name:        ParameterList::ParameterList
   ------------------------------------------------------------------------- */
ParameterList::ParameterList()
{
   pszList     = 0;
   listSize    = 0;
   pszMyParm   = 0;
   myParmSize  = 0;
   pszMyValue  = 0;
   myValueSize = 0;
   pszSOL      = 0;
}
/*ParameterList::ParameterList*/

/* -------------------------------------------------------------------------
   Name:        ParameterList::~ParameterList
   ------------------------------------------------------------------------- */
ParameterList::~ParameterList()
{
   delete [] pszList;
   delete [] pszMyParm;
   delete [] pszMyValue;
}
/*ParameterList::~ParameterList*/

/* -------------------------------------------------------------------------
   Name:        ParameterList::SetList
   ------------------------------------------------------------------------- */
void ParameterList::SetList (const char *pszValue)
{
   strclear (pszList);

   if (!pszValue)
      return;

   while (*pszValue) {
      if (*pszValue != '\r')
         charadd (pszList, *pszValue, listSize);
      pszValue++;
   }
}
/*ParameterList::SetList*/

/* -------------------------------------------------------------------------
   Name:        ParameterList::ClearParmList
   ------------------------------------------------------------------------- */
void ParameterList::ClearParmList()
{
   if (pszList)
      *pszList = 0;
}
/*ParameterList::ClearParmList*/

/* -------------------------------------------------------------------------
   Name:        ParameterList::AddParm
   ------------------------------------------------------------------------- */
bool ParameterList::AddParm (const char *pszName, const char *pszValue)
{
   if (pszList)
      stradd (pszList, "\n", listSize);

   stradd (pszList, pszName,  listSize);
   stradd (pszList, "=",      listSize);
   stradd (pszList, pszValue, listSize);
   return true;
}
/*ParameterList::AddParm*/

/* -------------------------------------------------------------------------
   Name:        ParameterList::RemoveParm
   ------------------------------------------------------------------------- */
bool ParameterList::RemoveParm (const char *pszName)
{
   bool  bFound;
   char *pszTemp;

   pszSOL  = pszList;
   pszTemp = pszSOL;
   bFound  = false;

   while (!bFound && pszSOL) {
      if (!GetParm())
         break;

      if (!strcmp (pszName, pszMyParm)) {     // Found parameter to be removed.
         bFound = true;
         if (pszSOL)
            // Shift the list up by one overwriting the item to be deleted.
            memcpy (pszTemp, pszSOL, listSize - (pszSOL - pszList));
         else {
            // Truncate the last record in the list.
            pszTemp--;
            *pszTemp = 0;
         }
      } else
         pszTemp = pszSOL;
   }
   return bFound;
}
/*ParameterList::RemoveParm*/

/* -------------------------------------------------------------------------
   Name:        ParameterList::GetParmCount
   ------------------------------------------------------------------------- */
int ParameterList::GetParmCount()
{
   int    count   = 0;
   char  *pszTemp = pszList;

   while (pszTemp) {
      count++;
      pszTemp = strchr (pszTemp, '\n');
      if (pszTemp)
         pszTemp++;
   }

   return count;
}
/*ParameterList::GetParmCount*/

/* -------------------------------------------------------------------------
   Name:        ParameterList::GetParm                             (private)
   ------------------------------------------------------------------------- */
bool ParameterList::GetParm()
{
   char  *pszEOL;     // Ptr to the end of line.
   char  *pszEqual;   // Ptr to the equal sign.
   char   stopper;

   if (!pszSOL)
      return false;

   // Clear the temporary variables.
   strclear (pszMyParm);
   strclear (pszMyValue);

   // Locate the EOL marker and temporarily replace with NUL.
   pszEOL = strchr (pszSOL, '\n');
   if (!pszEOL)
      pszEOL = strchr (pszSOL, '\0');
   stopper = *pszEOL;
   *pszEOL = '\0';

   // Locate the equal sign (=) separating the parameter and value.
   // If it isn't found, this is an error.
   pszEqual = strchr (pszSOL, '=');
   if (!pszEqual) {
      *pszEOL = stopper;
      return false;
   }

   // Extract the parameter and value.
   *pszEqual = '\0';
   stradd (pszMyParm, pszSOL, myParmSize);
   stradd (pszMyValue, pszEqual + 1, myValueSize);
   *pszEqual = '=';

   // Restore the EOL marker.
   *pszEOL = stopper;

   // Move to the next line (or set to NULL if no more lines).
   if (pszEOL && *pszEOL)
      pszSOL = pszEOL + 1;
   else
      pszSOL = 0;

   return true;
}
/*ParameterList::GetParm*/

/* -------------------------------------------------------------------------
   Name:        ParameterList::GetParmByName
   ------------------------------------------------------------------------- */
bool ParameterList::GetParmByName (const char *pszName, char *&pszValue)
{
   bool   bFound;

   // Clear the output variable.
   if (pszValue) {
      delete [] pszValue;
      pszValue = 0;
   }

   // Start searching for parameters at the beginning of the list.
   pszSOL = pszList;
   bFound = false;

   while (!bFound && pszSOL) {
      if (!GetParm())
         break;

      if (!strcmp (pszName, pszMyParm)) {           // Found desired parameter.
         bFound = true;
         pszValue = new char [strlen (pszMyValue) + 1];
         strcpy (pszValue, pszMyValue);
      }
   }
   return bFound;
}
/*ParameterList::GetParmByName*/

/* -------------------------------------------------------------------------
   Name:        ParameterList::GetParmByNumber
   ------------------------------------------------------------------------- */
bool ParameterList::GetParmByNumber (
   int    parmNo,
   char *&pszName,
   char *&pszValue)
{
   int  line = 0;

   // Clear the output variables.
   if (pszName) {
      delete [] pszName;
      pszName = 0;
   }
   if (pszValue) {
      delete [] pszValue;
      pszValue = 0;
   }

   if (parmNo < 1)
      return false;

   // Start searching for parameters at the beginning of the list.
   pszSOL = pszList;
   while (pszSOL && line < parmNo) {
      line++;
      GetParm();
   }

   if (line == parmNo) {
      pszName = new char [strlen (pszMyParm) + 1];
      strcpy (pszName, pszMyParm);
      pszValue = new char [strlen (pszMyValue) + 1];
      strcpy (pszValue, pszMyValue);
      return true;
   }

   return false;
}
/*ParameterList::GetParmByNumber*/

/* -------------------------------------------------------------------------
   Name:        ParameterList::DumpList
   ------------------------------------------------------------------------- */
void ParameterList::DumpList()
{
   printf ("<<<%s>>>", pszList);
}
/*ParameterList::DumpList*/
