#include <io.h>
#include <dir.h>

#include "..\BoardDLLs\brd_generic.h"

#include "speechtrainer.h"

#include <fstream>
using std::ofstream;

SpeechTrainer::SpeechTrainer(int portnum, char *pszAppName, GUILog* pGUILog, char* pszLogDir, bool bLogStatus)
{
   ivBoard = NULL;
   iPort = portnum;
   AppName = pszAppName;
   ivLog = pGUILog;
   LogDir = (pszLogDir != NULL) ? pszLogDir : "";
   currentTAMName = "";

   // retain the setting of IVRINFO.logSRAudio
   logSRAudio = bLogStatus;
   SRHandle = 0;
   unresolvedSRHandle = 0;

   Invalidate();
}

void SpeechTrainer::Invalidate()
{
   // initialise properties to empty
   retainAudio = false;
   SRAudioFile = "";
   SRHeard = "";
   SRAdjusted = "";
   SRConfidence = 0;
   SRReturn = IVR_NO_VOICE;   // start with no-voice as nothing has happened yet
   termDigit = '\0';
   grammarBase = "";
   grammarBaseData = "";
   grammarAdd = "";

   SRHandle++;                // previous recognition handle is now invalid;
                              // automatically start a new one
}

void SpeechTrainer::SetBoard(BoardGeneric* board)
{
   ivBoard = board;
}


void SpeechTrainer::UpdateTAMInfo(const char* pTAMName)
{
   currentTAMName = (pTAMName != NULL) ? pTAMName : "";
}

void SpeechTrainer::SetSRAcquireParm(const char* parameter)
{
   recognizerParameter = (parameter != NULL) ? parameter : "";
}

void SpeechTrainer::CaptureSpeechDetectTimes(const char* startdate, float starttime, const char* enddate, float endtime)
{
   speechStartDate = startdate;
   speechStartTime = starttime;
   speechEndDate = enddate;
   speechEndTime = endtime;
}

char* SpeechTrainer::ComposeFileName(int port, const char* extension)
{
   char* filename = NULL;
   SYSTEMTIME sysTime;

   if (logSRAudio)
   {
      filename = new char[48 + strlen(extension)];
      if(filename != NULL)
      {
         GetSystemTime(&sysTime);
         sprintf(filename, "SR audio %04d %02d %02d %02d.%02d.%02d.%04d port %03d.%s", sysTime.wYear, sysTime.wMonth, sysTime.wDay, sysTime.wHour, sysTime.wMinute, sysTime.wSecond, sysTime.wMilliseconds, port, extension);

         if(stricmp(extension, "wav") == 0)
         {
            SRAudioFile = filename;
         }
      }
   }
   else
   {
      SRAudioFile = "";
   }

   // TO DO:
   //    Don't store file name in SRAudioFile at this point
   //    Let the proposed new method to write the file also retain the name

   return(filename);
}
//SpeechTrainer::composeFileName


void SpeechTrainer::CapturePlayStats(const char* pszAppFile,
   int iStepNum, const char* pszStepName, const char* pszScriptcode,
   const char* pszFileName)
{
   const float  minHeardDuration = 0.700;    // seconds

   lastExecPlayStep.stepStat.TAPName  = pszAppFile;
   lastExecPlayStep.stepStat.TAMName  = currentTAMName;
   lastExecPlayStep.stepStat.stepNum  = iStepNum;
   lastExecPlayStep.stepStat.stepName = pszStepName;
   if(pszScriptcode != NULL)
   {
      lastExecPlayStep.script         = pszScriptcode;
   } else {
      lastExecPlayStep.script         = "";
   }
   lastExecPlayStep.file              = pszFileName;
   lastExecPlayStep.completed         = ivBoard->GetPlayCompleted();
   lastExecPlayStep.played_sec        = ivBoard->GetPlayElapsedTime();

   if((lastExecPlayStep.completed) || (lastExecPlayStep.played_sec >= minHeardDuration))
   {
      lastPlayedStep = lastExecPlayStep;
   }

   // QUESTION:
   //    If play completed, should we reset the SR values?
}
//SpeechTrainer::CapturePlayStats


void SpeechTrainer::CaptureSRStartStats(const char* pszAppFile,   // TAP name
                                   int iStepNum,                  // step number
                                   const char* pszStepName,       // Step name
                                   int iNoVoiceTimeout,           // no voice t/o
                                   unsigned int iEndVoiceTimeout, // end speech t/o
                                   unsigned int iThresholdConf,   // threshold confidence
                                   bool bRetainAudio,             // retainAudio
                                   Voice::SR_GRAMMAR srgType,     // grammar type
                                   const char* pszBaseGrammar,    // base grammar name/file
                                   const char* pszAddGrammar)     // additional grammar text
{
   int iGrammarIndex;

   lastSRStep.TAPName  = pszAppFile;
   lastSRStep.TAMName  = currentTAMName;
   lastSRStep.stepNum  = iStepNum;
   lastSRStep.stepName = pszStepName;

   novoiceTimeout = iNoVoiceTimeout;
   endvoiceTimeout = iEndVoiceTimeout;
   thresholdConf = iThresholdConf;
   retainAudio = bRetainAudio;

   grammarType = srgType;
   iGrammarIndex = 1;
   switch(grammarType)
   {
      case Voice::YES_NO :
         grammarBase = "Internal:YES_NO";
         break;

      case Voice::DIGITS :
         grammarBase = "Internal:DIGITS";
         break;

      case Voice::FILE :
         grammarBase = (pszBaseGrammar != NULL) ? pszBaseGrammar : "";
         // fall through

      case Voice::TEXT :
         if(pszBaseGrammar != NULL)
         {
            // Write grammar text to log dir, store name in grammarBase
            WriteGrammar(grammarType, pszBaseGrammar, iGrammarIndex);
            break;
         }
         // else fall through

      default :
         grammarBase = "";
   }

   // An additional grammar is always of type Voice::TEXT
   iGrammarIndex = 2;
   if(pszAddGrammar != NULL)
   {
      // Write grammar text to log dir, store name in grammarAdd
      WriteGrammar(Voice::TEXT, pszAddGrammar, iGrammarIndex);
   }
   else
   {
      grammarAdd = "";
   }

   // The SRAudioFile property is set by SpeechTrainer::ComposeFileName(),
   // which is only called when board level AudioRecognise has audio to process

   // Results are undefined until audio is processed by recognizer
   SRHeard = "";
   SRAdjusted = "";
   SRConfidence = 0;
}
//SpeechTrainer::CaptureSRStartStats


void SpeechTrainer::SetSRConceptHeard(const char* pszConcept)
{
   SRHeard = (pszConcept == NULL) ? "" : pszConcept;
   SRAdjusted = SRHeard;
}
//SpeechTrainer::SetSRConceptHeard


// Capture SR result for SpeechTrainer
void SpeechTrainer::CaptureSRResult(const char cDig,               // touch tone digit
                                    const char* pszConcept,        // result of recognition
                                    unsigned int iConfidence,      // confidence
                                    IV_STATUS status,              // result
                                    unsigned int* pHandle)         // to receive SR Handle
{
   char  timestr[12];

   // Capture result statistics
   termDigit = cDig;
   SRAdjusted = (pszConcept == NULL) ? "" : pszConcept;
   if(SRHeard.empty())
   {
      SRHeard = SRAdjusted;
   }
   SRConfidence = iConfidence;
   SRReturn = status;

   // Write the output file
   if (logSRAudio)
   {
      WriteSRData();    // report stats to file
   }

   if (pHandle != NULL)
   {
      // Remember a recognition as "unresolved" so we can flag it after the fact,
      // if it is determined to be wrong; this is to address false-positives
      // generated by certain SR API's

      //  SR handle:
      if(unresolvedSRHandle != 0)
      {
         ivLog->Print("         WARNING: Previous remembered recognition %u no longer stored, and is assumed to be valid\n", unresolvedSRHandle);
      }
      unresolvedSRHandle = SRHandle;
      *pHandle = unresolvedSRHandle;

      //  Call Start Date:
      unresolvedStartDate = ivBoard->GetCallStartDate();

      //  Call Start Time:
      sprintf(timestr, "%.3f", ivBoard->GetCallStartTime());
      unresolvedStartTime = timestr;

      //  SRAudioFile:
      unresolvedAudioFile = SRAudioFile;

      //  Port:
      //    same as current recognition
   }

   Invalidate();        // blank out stat properties for next recognition
}
//SpeechTrainer::CaptureSRResult


// Flag a previous SR result for review in SpeechTrainer GUI application
bool SpeechTrainer::FlagSRResult(unsigned int iHandle)
{
   string      FileName;
   char        datestr[12];
   char        timestr[12];
   char        portstr[4];
   char        tempstr[12];
   time_t      presentTime = time(NULL);
   struct tm*  tBlock = localtime(&presentTime);
   ofstream    flagFile;

   if(iHandle != unresolvedSRHandle)
   {
      ivLog->Print ("      ERROR: SR Handle %u does not correspond to a previous unresolved recognition.\n", iHandle);
      return(false);
   }

   strftime(datestr, 10, "%Y%m%d", tBlock);
   strftime(timestr, 10, "%H%M%S", tBlock);
   sprintf(portstr, "%i", iPort);

   FileName = ComposeTrainerPath();

   FileName += "st";
   FileName += datestr;
   FileName += "-";
   FileName += portstr;
   FileName += ".flg";

   // Open file
   flagFile.open(FileName.c_str(), std::ios::app);
   if(flagFile != NULL)
   {
      flagFile << "\"";                 // quote

      //  1  Call Start Date
      flagFile << unresolvedStartDate;
      flagFile << "\",";                // quote comma

      //  2  Call Start Time
      flagFile << unresolvedStartTime;
      flagFile << ",";                  // comma

      //  3  Port number
      flagFile << portstr;
      flagFile << ",\"";                // comma quote

      // 37  SRAudioFile
      flagFile << unresolvedAudioFile;
      flagFile << "\",";                // quote comma

      // 43  SR handle
      sprintf(tempstr, "%u", iHandle);
      flagFile << tempstr;

      flagFile << "\n";                 // newline

      flagFile.close();
      unresolvedSRHandle = 0;           // handle resolved, no longer "remembered"
      return(true);
   }
   else
   {
      ivLog->Print ("      ERROR: Failed to write to ST flag file '%s'\n", FileName.c_str());
      return(false);
   }
}
//SpeechTrainer::FlagSRResult


string SpeechTrainer::ComposeTrainerPath(void)
{
   string      Path = "";
   time_t      presentTime = time(NULL);
   struct tm*  tBlock = localtime(&presentTime);
   char        datestr[12];

   if (logSRAudio)
   {
      strftime(datestr, 10, "%Y%m%d", tBlock);
      Path = LogDir;
      Path += "st";
      Path += datestr;
      Path += "\\";

      // Create folder under log directory
      if(access(Path.c_str(), 0))
      {
         if(mkdir(Path.c_str()) != 0)
         {
            ivLog->Print ("      WARNING: SpeechTrainer failed to create folder '%s' \n", Path.c_str());
            Path = "";
         }
      }
   }
   return(Path);
}

// --------------------------------------------------------------------------
// Write grammar text to a secondary file
// --------------------------------------------------------------------------
void SpeechTrainer::WriteGrammar(Voice::SR_GRAMMAR srgType, const char* pszGrammar, int index)
{
   string      FilePath;
   char*       filename;
   char*       pos;
   char        extension[10];
   ofstream    grammarFile;


   // save the grammar name, in case we fail to copy it to a file
   if(index == 1)
   {
      grammarBaseData = (pszGrammar != NULL) ? pszGrammar : "";
   }
   else if(index == 2)
   {
      grammarAdd = (pszGrammar != NULL) ? pszGrammar : "";
   }
   else
   {
      ivLog->Print ("      WARNING: Invalid grammar index; SpeechTrainer data may be incorrect.\n");
   }

   sprintf(extension, "grmr%i", index);
   if(SRAudioFile.empty())
   {
      // Generate file name to store grammar text
      filename = ComposeFileName(iPort, extension);
   }
   else
   {
      // Use SRAudioFile name as base name,
      // remove extension (.wav) and replace with .grmr#
      filename = new char[SRAudioFile.length() + 10];
      strcpy(filename, SRAudioFile.c_str());
      pos = strrchr(filename, '.');
      if(pos != NULL)
      {
         strcpy(pos+1, extension);
      } else {
         strncat(filename, extension, 10);
      }
   }

   if(filename != NULL)
   {
      // Build full path to the file in the current log directory
      FilePath = ComposeTrainerPath();
      FilePath += filename;

      // Are we copying a FILE or writing TEXT?
      if(srgType == Voice::FILE)
      {
         // We've already verified that file exists for reading
         // Make a copy of the grammar to it our log folder
         if(!CopyFile(pszGrammar, FilePath.c_str(), false))
         {
            DWORD errorID = GetLastError();
            char MsgBuf[1024];
            FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, errorID, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), MsgBuf, 1024, 0);
            return;
         }
      }
      else
      {
         // Assume it is TEXT
         // Open file and write the grammar to it
         grammarFile.open(FilePath.c_str(), std::ios::app);
         if(grammarFile != NULL)
         {
            grammarFile.write(pszGrammar, strlen(pszGrammar));
            grammarFile.close();
         }
      }

      // Save the output file name (only) to appropriate grammar property
      if(index == 1)
      {
         grammarBaseData = filename;
      }
      else if(index == 2)
      {
         grammarAdd = filename;
      }
      else
      {
         ivLog->Print ("      WARNING: Invalid grammar index; SpeechTrainer data may be incorrect.\n");
      }

      // release memory used
      delete[] filename;
   }
}


// --------------------------------------------------------------------------
// Write SR trainer data to .csv file
// --------------------------------------------------------------------------
void SpeechTrainer::WriteSRData()
{
   string      FileName;
   char        datestr[12];
   char        timestr[12];
   char        portstr[4];
   char        tempstr[12];
   time_t      presentTime = time(NULL);
   struct tm*  tBlock = localtime(&presentTime);
   ofstream    trainerFile;

   strftime(datestr, 10, "%Y%m%d", tBlock);
   strftime(timestr, 10, "%H%M%S", tBlock);
   sprintf(portstr, "%i", iPort);

   FileName = ComposeTrainerPath();

   FileName += "st";
   FileName += datestr;
   FileName += "-";
   FileName += portstr;
   FileName += ".csv";

   // Open file
   trainerFile.open(FileName.c_str(), std::ios::app);
   if(trainerFile != NULL)
   {
      trainerFile << "\"";                  // quote

      //  1  Call Start Date
      trainerFile << ivBoard->GetCallStartDate();
      trainerFile << "\",";                // quote comma

      //  2  Call Start Time
      sprintf(timestr, "%.3f", ivBoard->GetCallStartTime());
      trainerFile << timestr;
      trainerFile << ",";                  // comma

      //  3  Port number
      trainerFile << portstr;
      trainerFile << ",\"";                // comma quote

      //  4  Application log file name
      trainerFile << AppName << ".log";
      trainerFile << "\",\"";              // quote comma quote

      //  5  PlayedStepTAP
      trainerFile << lastPlayedStep.stepStat.TAPName;
      trainerFile << "\",\"";              // quote comma quote

      //  6  PlayedStepTAM
      trainerFile << lastPlayedStep.stepStat.TAMName;
      trainerFile << "\",";                // quote comma

      //  7  PlayedStepNum
      trainerFile << lastPlayedStep.stepStat.stepNum;
      trainerFile << ",\"";                // comma quote

      //  8  PlayedStepName
      trainerFile << lastPlayedStep.stepStat.stepName;
      trainerFile << "\",\"";              // quote comma quote

      //  9  PlayedScript
      trainerFile << lastPlayedStep.script;
      trainerFile << "\",\"";              // quote comma quote

      // 10  PlayedFile
      trainerFile << lastPlayedStep.file;
      trainerFile << "\",\"";              // quote comma quote

      // 11  PlayedComplete
      trainerFile << ((lastPlayedStep.completed) ? "COMPLETED" : "INTERRUPTED");
      trainerFile << "\",";                // quote comma

      // 12  Playedto
      sprintf(tempstr, "%.3f", lastPlayedStep.played_sec);
      trainerFile << tempstr;
      trainerFile << ",\"";                // comma quote

      // 13  ExecStepTAP
      trainerFile << lastExecPlayStep.stepStat.TAPName;
      trainerFile << "\",\"";              // quote comma quote

      // 14  ExecStepTAM
      trainerFile << lastExecPlayStep.stepStat.TAMName;
      trainerFile << "\",";                // quote comma

      // 15  ExecStepNum
      trainerFile << lastExecPlayStep.stepStat.stepNum;
      trainerFile << ",\"";                // comma quote

      // 16   ExecStepName
      trainerFile << lastExecPlayStep.stepStat.stepName;
      trainerFile << "\",\"";              // quote comma quote

      // 17   ExecScript
      trainerFile << lastExecPlayStep.script;
      trainerFile << "\",\"";              // quote comma quote

      // 18   ExecFile
      trainerFile << lastExecPlayStep.file;
      trainerFile << "\",\"";              // quote comma quote

      // 19   ExecResult
      trainerFile << ((lastExecPlayStep.completed) ? "COMPLETED" : "INTERRUPTED");
      trainerFile << "\",";                // quote comma

      // 20  ExecTo
      sprintf(tempstr, "%.3f", lastExecPlayStep.played_sec);
      trainerFile << tempstr;
      trainerFile << ",\"";                // comma quote

      // 21  SRStepTAP
      trainerFile << lastSRStep.TAPName;
      trainerFile << "\",\"";              // quote comma quote

      // 22  SRStepTAM
      trainerFile << lastSRStep.TAMName;
      trainerFile << "\",";                // quote comma

      // 23  SRStepNum
      trainerFile << lastSRStep.stepNum;
      trainerFile << ",\"";                // comma quote

      // 24  SRStepName
      trainerFile << lastSRStep.stepName;
      trainerFile << "\",";                // quote comma

      // 25  NoVoiceTO
      trainerFile << novoiceTimeout;
      trainerFile << ",";                  // comma

      // 26  EndVoiceTO
      trainerFile << endvoiceTimeout;
      trainerFile << ",";                  // comma

      // 27  ThresholdConf
      trainerFile << thresholdConf;
      trainerFile << ",";                  // comma

      // 28  retainAudio
      trainerFile << retainAudio;
      trainerFile << ",\"";                // comma quote

      // 29  RecognizerParameters
      trainerFile << recognizerParameter;
      trainerFile << "\",\"";              // quote comma quote

      // 30  GrammarName
      trainerFile << grammarBase;
      trainerFile << "\",\"";              // quote comma quote

      // 31  GrammarBase
      trainerFile << grammarBaseData;
      trainerFile << "\",\"";              // quote comma quote

      // 32  GrammarAdd
      trainerFile << grammarAdd;
      trainerFile << "\",\"";              // quote comma quote

      if(SRAudioFile.empty())
      {
         // 33  SpeechStartDate
         trainerFile << "\",";             // quote comma
         // 34  SpeechStartTime
         trainerFile << ",\"";             // comma quote
         // 35  SpeechEndDate
         trainerFile << "\",";             // quote comma
         // 36  SpeechEndTime
         trainerFile << ",\"";             // comma quote
      }
      else
      {
         // 33  SpeechStartDate
         trainerFile << speechStartDate;
         trainerFile << "\",";             // quote comma

         // 34  SpeechStartTime
         sprintf(timestr, "%.3f", speechStartTime);
         trainerFile << timestr;
         trainerFile << ",\"";             // comma quote

         // 35  SpeechEndDate
         trainerFile << speechEndDate;
         trainerFile << "\",";             // quote comma

         // 36  SpeechEndTime (seconds since midnight)
         sprintf(timestr, "%.3f", speechEndTime);
         trainerFile << timestr;
         trainerFile << ",\"";             // comma quote
      }

      // 37  SRAudioFile
      trainerFile << SRAudioFile;
      trainerFile << "\",\"";              // quote comma quote

      // 38  SRHeard
      trainerFile << SRHeard;
      trainerFile << "\",\"";              // quote comma quote

      // 39  SRAdjusted
      trainerFile << SRAdjusted;
      trainerFile << "\",";                // quote comma

      // 40  SRResultConf
      trainerFile << SRConfidence;
      trainerFile << ",\"";                // comma quote

      // 41  SRTermDigit
      sprintf(tempstr, "%c", termDigit);
      trainerFile << tempstr;
      trainerFile << "\",\"";              // quote comma quote

      // 42  SRReturn
      switch(SRReturn)
      {
         case IVR_SUCCESS :
            trainerFile << "SUCCESS";
            break;

         case IVR_FAIL :
            trainerFile << "FAILURE";
            break;

         case IVR_FATAL :
            trainerFile << "FATAL";
            break;

         case IVR_TIMEOUT :
            trainerFile << "TIMEOUT";
            break;

         case IVR_HANGUP :
            trainerFile << "HANGUP";
            break;

         case IVR_THRESHOLD :
            trainerFile << "THRESHHOLD";
            break;

         default:
            sprintf(tempstr, "STATUS_%i", SRReturn);
            trainerFile << tempstr;
      }
      trainerFile << "\",";                // quote comma

      // 43  SR handle
      sprintf(tempstr, "%i", SRHandle);
      trainerFile << tempstr;

      trainerFile << "\n";               // newline

      trainerFile.close();
   }
   else
   {
      ivLog->Print ("      WARNING: SpeechTrainer failed to write to recognition file '%s'\n", FileName.c_str());
   }
}
//SpeechTrainer::WriteSRData

